// Copyright Matthias Troyer
// 2005. Distributed under the Boost Software License, Version
// 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/timer.hpp>
#include <iostream>
#include <fstream>

namespace test
{
  
  const std::size_t length=10000;
  
  double data[length];
  
  template <class T>
  const T& make_const(T& x)
  {
      return x;
  }
  
  double write_serialization(std::size_t number_of_repetitions)
  {
	  //my_stream output_stream;
	  std::ofstream output_stream("archive1");
	  boost::archive::binary_oarchive oa(output_stream);
	  
      boost::timer time;
	  while (number_of_repetitions--)
        oa << make_const(data);
	  output_stream.flush();
	  output_stream.close();
      return time.elapsed();
  }

  double write_direct(std::size_t number_of_repetitions)
  {
	  //my_stream output_stream;
	  std::ofstream output_stream("archive2");
	  
      boost::timer time;
 	  while (number_of_repetitions--) 
	    {
		  output_stream.write(reinterpret_cast<const char*>(&length), sizeof(std::size_t));
          output_stream.write(reinterpret_cast<const char*>(&(data[0])), length*sizeof(double));
	    }
      assert(output_stream.good());	  
	  output_stream.flush();
	  output_stream.close();
      return time.elapsed();
  }


  double read_serialization(std::size_t number_of_repetitions)
  {
	  //my_stream output_stream;
	  std::ifstream input_stream("archive1");
	  boost::archive::binary_iarchive ia(input_stream);
	  
      boost::timer time;
	  while (number_of_repetitions--)
        ia >> data;
	  input_stream.close();
      return time.elapsed();
  }

  double read_direct(std::size_t number_of_repetitions)
  {
	  //my_stream output_stream;
	  std::ifstream input_stream("archive2");
	  
      boost::timer time;
      std::size_t s;
 	  while (number_of_repetitions--) 
	    {
		  input_stream.read(reinterpret_cast<char*>(&s), sizeof(std::size_t));
          input_stream.read(reinterpret_cast<char*>(data), length*sizeof(double));
	    }
      assert(input_stream.good());	  
	  input_stream.close();
      return time.elapsed();
  }

}


int main()
{    

    const std::size_t number_of_repetitions=1000;
    std::cout
        << "serialization writing time:           "
        << test::write_serialization(number_of_repetitions)
        << std::endl;
    
    std::cout
        << "direct writing time: "
        << test::write_direct(number_of_repetitions)
        << std::endl;

    std::cout
        << "serialization reading time:           "
        << test::read_serialization(number_of_repetitions)
        << std::endl;
    
    std::cout
        << "direct reading time: "
        << test::read_direct(number_of_repetitions)
        << std::endl;


}

