// Copyright Matthias Troyer
// 2005. Distributed under the Boost Software License, Version
// 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <boost/archive/binary_oarchive.hpp>
#include <boost/archive/binary_iarchive.hpp>
#include <boost/timer.hpp>
#include <boost/iostreams/stream_buffer.hpp>
#include <boost/iostreams/stream.hpp>
#include <boost/iostreams/device/back_inserter.hpp>
#include <boost/serialization/vector.hpp>
#include <vector>
#include <iostream>
#include <fstream>
#include <string>

namespace test
{

  int live_code;
  
  namespace io = boost::iostreams;
  
  typedef io::stream< 
    io::back_insert_device
	  <std::string> > my_stream;

  double write_serialization(std::size_t number_of_elements)
  {
      const std::vector<double> data(number_of_elements);
	  //my_stream output_stream;
	  std::ofstream output_stream("archive1");
	  boost::archive::binary_oarchive oa(output_stream);
	  
      boost::timer time;
      oa << data;
	  output_stream.flush();
	  output_stream.close();
      return time.elapsed();
  }

  double write_direct(std::size_t number_of_elements)
  {
	  std::vector<double> data(number_of_elements);
	  //my_stream output_stream;
	  std::ofstream output_stream("archive2");
	  
      boost::timer time;
      std::size_t s = data.size();
      output_stream.write(reinterpret_cast<char*>(&s), sizeof(std::size_t));
      output_stream.write(reinterpret_cast<char*>(&(data[0])), number_of_elements*sizeof(double));
      assert(output_stream.good());	  
	  output_stream.flush();
	  output_stream.close();
      return time.elapsed();
  }

  double read_serialization()
  {
	  std::vector<double> data;
	  //my_stream output_stream;
	  std::ifstream input_stream("archive1");
	  boost::archive::binary_iarchive ia(input_stream);
	  
      boost::timer time;
      ia >> data;
	  input_stream.close();
      return time.elapsed();
  }

  double read_direct()
  {
	  std::vector<double> data;
	  //my_stream output_stream;
	  std::ifstream input_stream("archive2");
	  
      boost::timer time;
      std::size_t s;
      input_stream.read(reinterpret_cast<char*>(&s), sizeof(std::size_t));
	  data.resize(s);
      input_stream.read(reinterpret_cast<char*>(&(data[0])), s*sizeof(double));
      assert(input_stream.good());	  
	  input_stream.close();
      return time.elapsed();
  }


}


int main()
{    

    const std::size_t number_elements=10000000;
    std::cout
        << "serialization writing time:           "
        << test::write_serialization(number_elements)
        << std::endl;
    
    std::cout
        << "direct writing time: "
        << test::write_direct(number_elements)
        << std::endl;

    std::cout
        << "serialization reading time:           "
        << test::read_serialization()
        << std::endl;
    
    std::cout
        << "direct reading time: "
        << test::read_direct()
        << std::endl;


    // This is ultimately responsible for preventing all the test code
    // from being optimized away.  Change this to return 0 and you
    // unplug the whole test's life support system.
    return test::live_code < 0.;
}

